@@ -175,12 +175,6 @@ ot_indices <- function(x,
   if (!(is.data.frame(x) | is.matrix(x))) stop("`x` must be a matrix or a data.frame!")
   x <- as.data.frame(x)
 
-  # Check if the output is a numerical
-  if (!is.numeric(y) | !is.matrix(y)) stop("`y` must be a matrix of numerical values!")
-
-  # Check if the dimensions match
-  if (!(nrow(x) == nrow(y))) stop("The number of samples in `x` and `y` should be the same")
-
   # Check if the number of partitions is lower than the number of samples
   if (nrow(x) <= M) stop("The number of partitions should be lower than the number of samples")
 
@@ -188,12 +182,20 @@ ot_indices <- function(x,
   cost_type <- identical(cost, "L2")
   if (!cost_type & !is.function(cost)) stop("`cost` should be \"L2\" or a function")
 
+  # Check if the output is a numerical
+  if (cost_type & (!is.numeric(y) | !is.matrix(y)))
+    stop("`y` must be a matrix of numerical values!")
+
+  # Check if the dimensions match
+  if (!(nrow(x) == nrow(y)))
+    stop("The number of samples in `x` and `y` should be the same")
+
   # Check the logical values
   if (!is.logical(discrete_out)) stop("`discrete_out` should be logical")
   if (!is.logical(scaling)) stop("`scaling` should be logical")
 
   # Check if the solver is present in the pool
-  match.arg(solver, c("sinkhorn", "sinkhorn_stable", "transport"))
+  match.arg(solver, c("sinkhorn", "sinkhorn_log", "transport"))
 
   # Check that bootstrapping is correctly set
   if ((!boot & !is.null(R)) | (boot & is.null(R))) {
@@ -270,7 +272,7 @@ ot_indices <- function(x,
   solver_fun <- switch (
     solver,
     "sinkhorn" = sinkhorn,
-    "sinkhorn_stable" = sinkhorn_stable,
+    "sinkhorn_log" = sinkhorn_stable,
     "transport" = transport::transport,
     default = NULL
   )
@@ -308,7 +310,7 @@ ot_indices <- function(x,
     partition <- partitions[, k]
 
     # Set the number of partition elements
-    M <- max(partition)
+    M <- max(partition, na.rm = T)
 
     # NO BOOTSTRAP ESTIMATION
     if (!boot) {
@@ -458,7 +460,7 @@ ot_boot_discrete <- function(d,
 
   # Get the number of realizations and partitions
   N <- length(partition)
-  M <- max(partition)
+  M <- max(partition, na.rm = T)
 
   # Compute the unconditioned histogram
   a <- apply(y_unique, MARGIN = 1, FUN = function(u) sum(
@@ -523,7 +525,7 @@ ot_boot_cont <- function(d,
 
   # Get the number of realizations and partitions
   N <- length(partition)
-  M <- max(partition)
+  M <- max(partition, na.rm = T)
 
   # Build the histogram
   a <- rep(1 / N, N)
@@ -531,9 +533,6 @@ ot_boot_cont <- function(d,
   # Compute the variance
   V <- higher_bound(C) * scaling_param
 
-  # Get the number of partitions
-  M <- max(partition)
-
   # Initialize the return structure
   Wk <- matrix(nrow = 1, ncol = M)
   n <- matrix(nrow = M)
