@@ -111,7 +111,7 @@ ot_indices_wb <- function(x,
                                                 "original", "bias",
                                                 "low.ci", "high.ci"))))
     W_ci$input <- rep(names(W), times = 3)
-    W_ci$component <- rep(c("WB", "Advective", "Diffusive"), each = K)
+    W_ci$component <- rep(c("wass-bures", "advective", "diffusive"), each = K)
 
     IS_ci <- list()
     V_ci <- list()
@@ -124,7 +124,7 @@ ot_indices_wb <- function(x,
     partition <- partitions[, k]
 
     # Set the number of partition elements
-    M <- max(partition)
+    M <- max(partition, na.rm = T)
 
     # NO BOOTSTRAP ESTIMATION
     if (!boot) {
@@ -194,7 +194,7 @@ ot_indices_wb <- function(x,
   }
 
   if (boot) {
-    out <- gsaot_indices(method = "wasserstein-bures",
+    out <- gsaot_indices(method = "wass-bures",
                          indices = W,
                          bound = V,
                          IS = IS,
@@ -212,7 +212,7 @@ ot_indices_wb <- function(x,
     return(out)
   }
 
-  out <- gsaot_indices(method = "wasserstein-bures",
+  out <- gsaot_indices(method = "wass-bures",
                        indices = W,
                        bound = V,
                        IS = IS,
@@ -246,7 +246,7 @@ ot_wb_boot <- function(d,
   V <- 2 * traceCy
 
   # Get the number of partitions
-  M <- max(partition)
+  M <- max(partition, na.rm = T)
 
   # Initialize the return structure
   Wk <- matrix(nrow = 3, ncol = M)
